import java.awt.*;
import java.awt.geom.*;

/**
 * A rectangle that can be manipulated and drawn.
 * @author Joel Ross
 * @author adapted from Michael Kolling and David J. Barnes
 * @version Aug 2013
 *
 */
public class Rectangle extends DrawableShape
{
    //instance variables
    private int x;
    private int y;
    private int width;
    private int height;
    private String color;
    private boolean filled;

    /**
     * Creates a new Rectangle object with default location, radius, and color.
     */
    public Rectangle()
    {
        //give instance variables default values
        this.x = 60;
        this.y = 50;
        this.width = 50;
        this.height = 30;
        this.color = "red";
        this.filled = true;

        this.addSelfToCanvas();
    }

    /**
     * Returns the rectangle's current x position
     */
    public int getXPosition()
    {
        return x;
    }

    /**
     * Sets the rectangle's x position to the given value
     */
    public void setXPosition(int x)
    {
        this.x = x;
    }

    /**
     * Returns the rectangle's current y position
     */
    public int getYPosition()
    {
        return y;
    }

    /**
     * Sets the rectangle's y position to the given value
     */
    public void setYPosition(int y)
    {
        this.y = y;
    }

    /**
     * Returns the rectangle's current width
     */
    public int getWidth()
    {
        return width;
    }

    /**
     * Sets the rectangle's width to the given value
     */
    public void setWidth(int width)
    {
        this.width = width;
    }

    /**
     * Returns the rectangle's current height
     */
    public int getHeight()
    {
        return height;
    }

    /**
     * Sets the rectangle's height to the given value
     */
    public void setHeight(int height)
    {
        this.height = height;
    }

    /**
     * Returns the rectangle's current color (as a String). Can either be: 
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public String getColorName()
    {
        return color;
    }

    /**
     * Set the circle's color to the given value. Can either be:
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public void setColorByName(String color)
    {
        this.color = color;
    }

    /**
     *Returns whether the rectangle is filled or not
     */
    public boolean isFilled()
    {
        return filled;
    }

    /**
     * Sets whether the rectangle should be filled; can either be true or false
     */
    public void setFilled(boolean filled)
    {
        this.filled = filled;
    }

    /**
     * Move the rectangle horizontally the specified number of pixels
     */
    public void moveHorizontal(int distance)
    {
        x += distance;
    }

    /**
     * Move the rectangle vertically the specified number of pixels
     */
    public void moveVertical(int distance)
    {
        y += distance;
    }

    /**
     * Removes the shape from the canvas
     * Wrapper around parent class
     */
    public void removeSelfFromCanvas()
    {
        super.removeSelfFromCanvas();
    }

    /**
     * Sends the shape to the "back" of the drawing (behind other shapes)
     */
    public void sendToBack()
    {
        super.sendToBack();
    }

    /**
     * Brings the shape to the "front" of the drawing (in front of other shapes)
     */
    public void bringToFront()
    {
        super.bringToFront();
    }

    
    /* (non-Javadoc)
     * @see DrawableShape#draw(java.awt.Graphics2D)
     */
    public void draw(Graphics2D g2d)
    {
        g2d.setColor(getColorFromString(color));
        if(filled)
            g2d.fillRect(x,y,width,height);
        else
            g2d.drawRect(x,y,width,height);
    }
}
