import java.awt.*;
import java.awt.geom.*;

/**
 * An equilateral triangle that can be manipulated and drawn.
 * @author Joel Ross
 * @author adapted from Michael Kolling and David J. Barnes
 * @version Aug 2013
 *
 */
public class Triangle extends DrawableShape
{
    //instance variables
    private int x;
    private int y;
    private int sideLength;
    private int rotation;
    private String color;
    private boolean filled;

    /**
     * Creates a new Triangle object with default location, radius, and color.
     */
    public Triangle()
    {
        //give instance variables default values
        this.x = 40;
        this.y = 50;
        this.sideLength = 30;
        this.rotation = 0;
        this.color = "green";
        this.filled = true;

        this.addSelfToCanvas();
    }

    /**
     * Returns the triangle's current x position
     */
    public int getXPosition()
    {
        return x;
    }

    /**
     * Sets the triangle's x position to the given value
     */
    public void setXPosition(int x)
    {
        this.x = x;
    }

    /**
     * Returns the triangle's current y position
     */
    public int getYPosition()
    {
        return y;
    }

    /**
     * Sets the triangle's x position to the given value
     */
    public void setYPosition(int y)
    {
        this.y = y;
    }

    /**
     * Returns the triangle's current side length
     */
    public int getSideLength()
    {
        return sideLength;
    }

    /**
     * Sets the triangles's side length to the given value
     */
    public void setSideLength(int sideLength)
    {
        this.sideLength = sideLength;
    }

    /**
     * Returns the triangle's current rotation (in radians)
     */
    public double getRotation()
    {
        return rotation;
    }

    /**
     * Set the triangle's current rotation (in degrees). Rotation is CLOCKWISE
     */
    public void setRotation(int rotation)
    {
        this.rotation = rotation;
    }

    /**
     * Returns the triangle's current color (as a String). Can either be: 
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public String getColorName()
    {
        return color;
    }

    /**
     * Set the circle's color to the given value. Can either be:
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public void setColorByName(String color)
    {
        this.color = color;
    }

    /**
     * Returns whether the triangle is filled or not
     */
    public boolean isFilled()
    {
        return filled;
    }

    /**
     * Sets whether the triangle is filled; can either be true or false
     */
    public void setFilled(boolean filled)
    {
        this.filled = filled;
    }

    /**
     * Move the triangle horizontally by the specified number of pixels.
     */
    public void moveHorizontal(int distance)
    {
        x += distance;
    }

    /**
     * Move the triangle horizontally by the specified number of pixels.
     */
    public void moveVertical(int distance)
    {
        y += distance;
    }

    /**
     * Removes the shape from the canvas
     * Wrapper around parent class
     */
    public void removeSelfFromCanvas()
    {
        super.removeSelfFromCanvas();
    }

    /**
     * Sends the shape to the "back" of the drawing (behind other shapes)
     */
    public void sendToBack()
    {
        super.sendToBack();
    }

    /**
     * Brings the shape to the "front" of the drawing (in front of other shapes)
     */
    public void bringToFront()
    {
        super.bringToFront();
    }

    
    /* (non-Javadoc)
     * @see DrawableShape#draw(java.awt.Graphics2D)
     */
    public void draw(Graphics2D g2d)
    {
        g2d.setColor(getColorFromString(color));

        double height = Math.sqrt(3)/2.0*sideLength; //the height of the triangle
        int[] xPoints = {x, x+sideLength/2, x+sideLength}; //get the coordinates of the corners of the triangle
        int[] yPoints = {(int)(y+height), y, (int)(y+height)};

        double radians = rotation*Math.PI/180.0; //convert to radians
        g2d.rotate(radians, x+sideLength/2, y+sideLength/2); //rotate the context around the center (converting to radians)

        if(filled)
            g2d.fill(new Polygon(xPoints, yPoints, 3));
        else
            g2d.draw(new Polygon(xPoints, yPoints, 3));

        g2d.rotate(-radians, x+sideLength/2, y+sideLength/2); //unrotate the context (again around the center)

    }
}
